/***************************************************************************
 *   Copyright (C) 2005 by Andreas Theofilu   *
 *   andreas@TheosSoft.net   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <malloc.h>
#include <unistd.h>
#include <stdarg.h>
#include "language.h"

int lang;
char slang[11];
char lfile[512];

int ReadLine (int fd, char *buf, int len);

void InitLang ()
{
char *p;

	lang = 0;

	if ((p = getenv ("LANG")) != NULL)
	{
	   if (!strncasecmp (p, "de", 2) || !strncasecmp (p, "de-at", 5) ||
               !strncasecmp (p, "at", 2) || !strncasecmp (p, "ch", 2))
              lang = 1;

           if (!strncasecmp (p, "fr", 2))
              lang = 2;
        }

	switch (lang)
	{
	   case 0: strcpy (slang, "en"); break;
	   case 1: strcpy (slang, "de"); break;
	   case 2: strcpy (slang, "fr"); break;
	   default: strcpy (slang, "C"); break;
	}

#ifdef WINDBLOWS
	if ((p = getenv ("SYSTEMROOT")) == NULL)
	   p = "C:";

	sprintf (lfile, "%s\\WinHandy_%s.txt", p, slang);

	if (access (lfile, R_OK) == -1)
	{
	   sprintf (lfile, "%s\\system32\\WinHandy_%s.txt", p, slang);
	   
	   if (access (lfile, R_OK) == -1)
	   {
	      sprintf (lfile, ".\\WinHandy_%s.txt", slang);
	      
	      if (access (lfile, R_OK) == -1)
	         lfile[0] = 0;
  	   }
	}
#else
	if ((p = getenv ("HOME")) == NULL)
	   p = ".";

	sprintf (lfile, "/usr/local/share/handy/handy_%s.txt", slang);
	
	if (access (lfile, R_OK) == -1)
	{
	   sprintf (lfile, "/usr/share/handy/handy_%s.txt", slang);
	   
	   if (access (lfile, R_OK) == -1)
	   {
	      sprintf (lfile, "%s/handy_%s.txt", p, slang);
	      
	      if (access (lfile, R_OK) == -1)
	      {
	         sprintf (lfile, "./handy_%s.txt", slang);
		 
		 if (access (lfile, R_OK) == -1)
	  	    lfile[0] = 0;
	      }
  	   }
	}
#endif
}

void lprintf (int num, char *txt, ...)
{
va_list ap;
int fd, nb;
char buffer[1024], *p;

	/*
	 * Wenn wir eine Textdatei haben, versuchen wir diese zu oeffnen.
	 * Sofern sie existiert (wenn ueberhaupt nur unter *NIX-Systemen),
	 * versuchen wir den gewuenschten Text aus dieser Datei zu ziehen.
	 * Gibt es die Datei nicht oder den gewuenschten Text, dann wird der
	 * im Parameter uebergebene Text ausgegeben.
	 */
	if (lfile)
	{
	   if ((fd = open (lfile, O_RDONLY, 0600)) != -1)
	   {
	      while (ReadLine (fd, &buffer[0], sizeof (buffer)) > 0)
	      {
	         if (buffer[0] == '#' || (p = strchr (buffer, ':')) == NULL)
	            continue;

		 *p = 0;
		 nb = atoi (buffer);

		 if (nb == num)
		 {
		    p++;
		    va_start (ap, txt);
		    vfprintf (stdout, p, ap);
		    va_end (ap);
		    close (fd);
		    return;
		 }
	      }

	      close (fd);
	   }
	}
	
	va_start (ap, txt);
	vfprintf (stdout, txt, ap);
	va_end (ap);
}

void lfprintf (int num, FILE *fp, char *txt, ...)
{
va_list ap;
int fd, nb;
char buffer[1024], *p;

	/*
	 * Wenn wir eine Textdatei haben, versuchen wir diese zu oeffnen.
	 * Sofern sie existiert (wenn ueberhaupt nur unter *NIX-Systemen),
	 * versuchen wir den gewuenschten Text aus dieser Datei zu ziehen.
	 * Gibt es die Datei nicht oder den gewuenschten Text, dann wird der
	 * im Parameter uebergebene Text ausgegeben.
	 */
	if (lfile)
	{
	   if ((fd = open (lfile, O_RDONLY, 0600)) != -1)
	   {
	      while (ReadLine (fd, &buffer[0], sizeof (buffer)) > 0)
	      {
	         if (buffer[0] == '#' || (p = strchr (buffer, ':')) == NULL)
	            continue;
	            
		 *p = 0;
		 nb = atoi (buffer);

		 if (nb == num)
		 {
		    p++;
		    va_start (ap, txt);
		    vfprintf (fp, p, ap);
		    va_end (ap);
		    close (fd);
		    return;
		 }
	      }

	      close (fd);
	   }
	}

	va_start (ap, txt);
	vfprintf (fp, txt, ap);
	va_end (ap);
}

int ReadLine (int fd, char *buf, int len)
{
unsigned char ch;
char *p;
int i, flag;

	p = buf;
	i = flag = 0;

	while (read (fd, &ch, 1) >= 1)
	{
	   if (ch == 0x0a)
	   {
	      *p = 0;
	      return strlen (buf);
           }

	   if (flag)
	   {
	      switch (ch)
	      {
	         case 'r': *p = '\r'; break;
	         case 'n': *p = '\n'; break;
	         case 'f': *p = '\f'; break;
	         case '\\': *p = '\\'; break;
	         default: *p = ch;
  	      }
  	      
  	      flag = 0;
  	      p++;
  	      i++;
  	      continue;
 	   }
 
	   if (ch == '\\')
	   {
	      flag = 1;
	      continue;
	   }

           if (ch == 0x0d)
              *p = 0;
           else
              *p = ch;

           if (i < len)
              p++;

           i++;
	}

	*p = 0;
	return strlen (buf);
}
